@@ -23,9 +23,11 @@ import os.path
 import subprocess
 import sys
 
+import yaml
+
 from .core import WorkspaceInstance
 from .mgmt import get_local_config, add_key, add_ssh_path, list_local_keys
-from .mgmt import modify_local
+from .mgmt import find_wd, modify_local
 from .api import HSAPIClient
 
 
@@ -39,6 +41,28 @@ loghandler.setFormatter(logging.Formatter('%(name)s.%(funcName)s (%(levelname)s)
 logger.addHandler(loghandler)
 
 
+def get_config_with_index(id_prefix=None):
+    try:
+        config = get_local_config()
+    except:
+        print('error loading configuration data. does it exist?')
+        return None, None, 1
+    if len(config['wdeployments']) == 0:
+        print(('ERROR: no workspace deployment in local configuration.'))
+        return config, None, 1
+    if id_prefix:
+        index = find_wd(config, id_prefix)
+        if index is None:
+            print('ERROR: given prefix does not match precisely 1 workspace deployment')
+            return config, None, 1
+    else:
+        if len(config['wdeployments']) > 1:
+            print('ERROR: ambiguous command: more than 1 workspace deployment defined.')
+            return config, None, 1
+        index = 0
+    return config, index, 0
+
+
 def main(argv=None):
     if argv is None:
         argv = sys.argv[1:]
@@ -50,6 +74,11 @@ def main(argv=None):
     argparser.add_argument('-v', '--verbose', action='store_true', default=False,
                            help='print verbose messages about actions by the hardshare client',
                            dest='verbose')
+    argparser.add_argument('--format', metavar='FORMAT',
+                           default=None, type=str,
+                           help=('special output formatting (default is no special formatting); '
+                                 'options: YAML , JSON'),
+                           dest='output_format')
     argparser.add_argument('-s', '--server-name', default='hs.rerobots.net',
                            help='name or IP address of hardshare server',
                            dest='server_name')
@@ -71,6 +100,11 @@ def main(argv=None):
     config_parser = subparsers.add_parser('config',
                                           description=config_commanddesc,
                                           help=config_commanddesc)
+    config_parser.add_argument('id_prefix', metavar='ID', nargs='?', default=None,
+                               help=('id of workspace deployment for configuration changes'
+                                     ' (can be unique prefix); '
+                                     'this argument is not required '
+                                     'if there is only 1 workspace deployment'))
     config_parser.add_argument('-c', '--create', action='store_true', default=False,
                                dest='create_config',
                                help='if no local configuration is found, then create one')
@@ -83,6 +117,9 @@ def main(argv=None):
     config_parser.add_argument('--add-raw-device', metavar='PATH', type=str,
                                dest='raw_device_path', default=None,
                                help='add device file to present in container')
+    config_parser.add_argument('--assign-image', metavar='IMG', type=str,
+                               dest='cprovider_img', default=None,
+                               help='assign image for cprovider to use (advanced option)')
     config_parser.add_argument('--rm-raw-device', metavar='PATH', type=str,
                                dest='remove_raw_device_path', default=None,
                                help='remove device previously marked for inclusion in container')
@@ -131,11 +168,19 @@ def main(argv=None):
     status_parser = subparsers.add_parser('status',
                                           description=status_commanddesc,
                                           help=status_commanddesc)
+    status_parser.add_argument('id_prefix', metavar='ID', nargs='?', default=None,
+                               help=('id of target workspace deployment'
+                                     ' (can be unique prefix)'))
 
     advertise_commanddesc = 'advertise availability, accept new instances'
     advertise_parser = subparsers.add_parser('ad',
                                              description=advertise_commanddesc,
                                              help=advertise_commanddesc)
+    advertise_parser.add_argument('id_prefix', metavar='ID', nargs='?', default=None,
+                                  help=('id of workspace deployment to advertise'
+                                        ' (can be unique prefix); '
+                                        'this argument is not required '
+                                        'if there is only 1 workspace deployment'))
     advertise_parser.add_argument('-d', '--daemon', action='store_true', default=False,
                                   help='detach from invoking terminal (i.e., run as daemon)',
                                   dest='become_daemon')
@@ -144,6 +189,9 @@ def main(argv=None):
     terminate_parser = subparsers.add_parser('terminate',
                                              description=terminate_commanddesc,
                                              help=terminate_commanddesc)
+    terminate_parser.add_argument('id_prefix', metavar='ID', nargs='?', default=None,
+                                  help=('id of target workspace deployment'
+                                        ' (can be unique prefix)'))
     terminate_parser.add_argument('-f', '--force', action='store_true', default=False,
                                   help=('if there is an active instance, then'
                                         ' stop it without waiting'),
@@ -190,6 +238,14 @@ def main(argv=None):
     if argv_parsed.verbose:
         logger.setLevel(logging.DEBUG)
 
+    if argv_parsed.output_format is not None:
+        output_format = argv_parsed.output_format.lower()
+        if output_format not in ['yaml', 'json']:
+            print('output format unrecognized: {}'.format(argv_parsed.output_format))
+            return 1
+    else:
+        output_format = None
+
     try:
         ac = HSAPIClient(server_name=argv_parsed.server_name,
                          server_port=argv_parsed.server_port,
@@ -199,44 +255,53 @@ def main(argv=None):
 
     if argv_parsed.command == 'status':
         try:
-            config = get_local_config(collect_errors=True)
+            config = get_local_config()
         except:
             print('error loading configuration data. does it exist?')
             return 1
-        if len(config['wdeployments']) == 0:
-            findings = WorkspaceInstance.inspect_instance()
+        if argv_parsed.id_prefix is None:
+            if len(config['wdeployments']) == 0:
+                findings = [WorkspaceInstance.inspect_instance()]
+            else:
+                findings = []
+                for wd in config['wdeployments']:
+                    findings.append(WorkspaceInstance.inspect_instance(wdeployment=wd))
         else:
-            findings = WorkspaceInstance.inspect_instance(wdeployment=config['wdeployments'][0])
-        print(json.dumps(findings, indent=4))
+            findings = []
+            for m in find_wd(config, argv_parsed.id_prefix, one_or_none=False):
+                findings.append(WorkspaceInstance.inspect_instance(wdeployment=config['wdeployments'][m]))
+
+        if output_format == 'json':
+            print(json.dumps(findings))
+        else:  # output_format == 'yaml'
+            print(yaml.dump(findings, default_flow_style=False))
 
     elif argv_parsed.command == 'ad':
         if ac is None:
             print('cannot register without initial local configuration.'
                   ' (try `hardshare config --create`)')
             return 1
+        config, index, rc = get_config_with_index(argv_parsed.id_prefix)
+        if rc != 0:
+            return rc
         if argv_parsed.become_daemon:
             if os.fork() != 0:
                 return 0
             os.close(0)
             os.close(1)
             os.close(2)
-        return ac.run_sync()
+        return ac.run_sync(config['wdeployments'][index]['id'])
 
     elif argv_parsed.command == 'terminate':
+        config, index, rc = get_config_with_index(argv_parsed.id_prefix)
+        if rc != 0:
+            return rc
         if argv_parsed.purge_supposed_instance:
-            try:
-                config = get_local_config()
-            except:
-                print('error loading configuration data. does it exist?')
-                return 1
-            if len(config['wdeployments']) == 0:
-                print('ERROR: no workspace deployment defined! Try: hardshare check')
-                return 1
-            cprovider = config['wdeployments'][0]['cprovider']
+            cprovider = config['wdeployments'][index]['cprovider']
             if cprovider not in ['docker', 'podman']:
                 print('unknown cprovider: {}'.format(cprovider))
                 return 1
-            findings = WorkspaceInstance.inspect_instance(wdeployment=config['wdeployments'][0])
+            findings = WorkspaceInstance.inspect_instance(wdeployment=config['wdeployments'][index])
             if 'container' in findings:
                 try:
                     subprocess.check_call([cprovider, 'rm', '-f',
@@ -254,7 +319,7 @@ def main(argv=None):
             if ac is None:
                 print('cannot terminate without valid API client')
                 return 1
-            ac.terminate()
+            ac.terminate(config['wdeployments'][index]['id'])
             return 0
 
     elif argv_parsed.command == 'register':
@@ -320,29 +385,6 @@ def main(argv=None):
                       ' does it exist? is it broken?')
                 return 1
 
-            print('workspace deployments defined in local configuration:')
-            if len(config['wdeployments']) == 0:
-                print('\t(none)')
-            else:
-                for wdeployment in config['wdeployments']:
-                    print('{}\n\towner: {}\n\tcprovider: {}\n\tcargs: {}\n\timg: {}'.format(
-                        wdeployment['id'],
-                        wdeployment['owner'],
-                        wdeployment['cprovider'],
-                        wdeployment['cargs'],
-                        wdeployment['image'],
-                    ))
-
-            print('\nfound keys:')
-            if len(config['keys']) == 0:
-                print('\t(none)')
-            else:
-                print('\t' + '\n\t'.join(config['keys']))
-            if 'err_keys' in config and len(config['err_keys']) > 0:
-                print('found possible keys with errors:')
-                for err_key_path, err in config['err_keys']:
-                    print('\t {}: {}'.format(err, err_key_path))
-
             if not argv_parsed.only_local_config:
                 # Try to get remote config, given possibly new local config
                 try:
@@ -352,26 +394,69 @@ def main(argv=None):
                     print('Error occurred while contacting remote server '
                           'at {}'.format(ac.base_uri))
                     return 1
-                if 'err' in remote_config:
-                    print('Error occurred while contacting remote server.')
-                    if remote_config['err'] == 'wrong authorization token':
-                        print('wrong API token. Did it expire?')
-                    else:
-                        print(remote_config['err'])
-                    return 1
-                if len(remote_config['deployments']) == 0:
-                    print('\nno registered workspace deployments with this user account')
+
+                config = {
+                    'local': config,
+                    'remote': remote_config,
+                }
+
+            if output_format == 'json':
+                print(json.dumps(config))
+
+            elif output_format == 'yaml':
+                print(yaml.dump(config, default_flow_style=False))
+
+            else:
+                if 'local' not in config:
+                    config = {
+                        'local': config,
+                        'remote': None,
+                    }
+                print('workspace deployments defined in local configuration:')
+                if len(config['local']['wdeployments']) == 0:
+                    print('\t(none)')
                 else:
-                    print('\nregistered workspace deployments with this user account:')
-                    for wd in remote_config['deployments']:
-                        print('{}'.format(wd['id']))
-                        print('\tcreated: {}'.format(wd['date_created']))
-                        print('\torigin (address) of registration: {}'
-                              .format(wd['origin']))
+                    for wdeployment in config['local']['wdeployments']:
+                        print('{}\n\turl: {}\n\towner: {}\n\tcprovider: {}\n\tcargs: {}\n\timg: {}'.format(
+                            wdeployment['id'],
+                            'https://rerobots.net/workspace/{}'.format(wdeployment['id']),
+                            wdeployment['owner'],
+                            wdeployment['cprovider'],
+                            wdeployment['cargs'],
+                            wdeployment['image'],
+                        ))
+
+                print('\nfound keys:')
+                if len(config['local']['keys']) == 0:
+                    print('\t(none)')
+                else:
+                    print('\t' + '\n\t'.join(config['local']['keys']))
+                if 'err_keys' in config['local'] and len(config['local']['err_keys']) > 0:
+                    print('found possible keys with errors:')
+                    for err_key_path, err in config['local']['err_keys'].items():
+                        print('\t {}: {}'.format(err, err_key_path))
+
+                if config['remote']:
+                    if 'err' in config['remote']:
+                        print('Error occurred while contacting remote server.')
+                        if config['remote']['err'] == 'wrong authorization token':
+                            print('wrong API token. Did it expire?')
+                        else:
+                            print(config['remote']['err'])
+                        return 1
+                    if len(config['remote']['deployments']) == 0:
+                        print('\nno registered workspace deployments with this user account')
+                    else:
+                        print('\nregistered workspace deployments with this user account:')
+                        for wd in config['remote']['deployments']:
+                            print('{}'.format(wd['id']))
+                            print('\tcreated: {}'.format(wd['date_created']))
+                            print('\torigin (address) of registration: {}'
+                                  .format(wd['origin']))
 
         elif argv_parsed.prune_err_keys:
             _, errored_keys = list_local_keys(collect_errors=True)
-            for err_key_path, err in errored_keys:
+            for err_key_path, err in errored_keys.items():
                 print('deleting {}...'.format(err_key_path))
                 os.unlink(err_key_path)
 
@@ -402,16 +487,10 @@ def main(argv=None):
             ac.sync_config()
 
         elif argv_parsed.raw_device_path is not None:
-            config = get_local_config()
-            if len(config['wdeployments']) == 0:
-                print(('ERROR: Cannot add device because '
-                       'no workspace deployment in local configuration.'))
-                return 1
-            elif len(config['wdeployments']) > 1:
-                print('ERROR: ambiguous command: more than 1 workspace deployment defined.')
-                print('Future versions of hardshare client will better support this case.')
-                return 1
-            cprovider = config['wdeployments'][0]['cprovider']
+            config, index, rc = get_config_with_index(argv_parsed.id_prefix)
+            if rc != 0:
+                return rc
+            cprovider = config['wdeployments'][index]['cprovider']
             if cprovider not in ['docker', 'podman']:
                 print('unknown cprovider: {}'.format(cprovider))
                 return 1
@@ -419,21 +498,37 @@ def main(argv=None):
                 print('ERROR: given device file does not exist')
                 return 1
             carg = '--device={D}:{D}'.format(D=argv_parsed.raw_device_path)
-            config['wdeployments'][0]['cargs'].append(carg)
+            config['wdeployments'][index]['cargs'].append(carg)
             modify_local(config)
 
         elif argv_parsed.remove_raw_device_path is not None:
-            config = get_local_config()
-            if len(config['wdeployments']) == 0:
-                print('ERROR: no workspace deployment in local configuration.')
-                return 1
-            elif len(config['wdeployments']) > 1:
-                print('ERROR: ambiguous command: more than 1 workspace deployment defined.')
-                return 1
+            config, index, rc = get_config_with_index(argv_parsed.id_prefix)
+            if rc != 0:
+                return rc
             carg = '--device={D}:{D}'.format(D=argv_parsed.remove_raw_device_path)
-            config['wdeployments'][0]['cargs'].remove(carg)
+            config['wdeployments'][index]['cargs'].remove(carg)
             modify_local(config)
 
+        elif argv_parsed.cprovider_img is not None:
+            config, index, rc = get_config_with_index(argv_parsed.id_prefix)
+            if rc != 0:
+                return rc
+            cprovider = config['wdeployments'][index]['cprovider']
+            if cprovider not in ['docker', 'podman']:
+                print('unknown cprovider: {}'.format(cprovider))
+                return 1
+            cp_images = subprocess.run([cprovider, 'image', 'exists', argv_parsed.cprovider_img])
+            if cp_images.returncode != 0:
+                print('ERROR: given image name is not recognized by cprovider')
+                return 1
+            config['wdeployments'][index]['image'] = argv_parsed.cprovider_img
+            modify_local(config)
+
+        else:
+            print('Use `hardshare config` with a switch. For example, `hardshare config -l`')
+            print('or to get a help message, enter\n\n    hardshare help config')
+            return 1
+
     return 0
 
 
