@@ -33,6 +33,20 @@ logger = logging.getLogger(__name__)
 WEBUI_PUBLIC_KEY = """ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAACAQCpaC+WZHknryjjioYHcCpVweekg7lTFoy4Qc+je5YTJwCEt9NuEQis0Y5pGkVVNWCVTsrA95Cq/H67Sls5dvxul/R20hzmKVaWBxN6QIHxJAqB7z454pNvGRrLOi/PLBeImNgvOC4d5L4uYQyFVpBIRBSbRNaqV9zbfSIFFBjfqIv6Cp4SvvFu0rtHw4A/H62Firy+sap5nkcTynOSe3OskSItmAQT5I42nN6QfjKf9Y2MY0iFbP4GixGdvi0Nre0lRqVV8h5OQwGcrzfSQKBAeWASmJ2fyXja/WBk0UZFx0GSfNEP201xJbx2b9TVZMekBvJpYyDzc2nR/1kddLWucvE3malTW7Geuih5/hjPALXsqFL/zDZ89AF3DsY/QX2cSFazXBVVUz/UdSCmddZMtnL589sWNB2DkUldJXLkCNxGKlKh9aWoH/a6b3AR0EU67RyKK0wSMhcBPGcHLa2Zz/iaFJyNfEmeEhZ8KFz0oZ34cLEwtQRHYI7LtjT/ZT/5AamkGldQ4lMAFOdfc1Qu2LeOpYT9FOF+mkuT7MMlCpWNBR48+AVjni+uoNymStYBDapxPHDJ2D9TjgMDzKgBl+VqwNag8A532dgBxIZsjrqfMMQC7VTUVoeRXgOM8D0FZWYRmw6ozrYsJVaLE7OSqi9JKi50fFCoAMsbRKeFZw== scott@cero"""
 
 
+def find_wd(config, id_prefix, one_or_none=True):
+    matches = []
+    for ii, wd in enumerate(config['wdeployments']):
+        if wd['id'].startswith(id_prefix):
+            if one_or_none and len(matches) >= 1:
+                return None
+            matches.append(ii)
+    if not one_or_none:
+        return matches
+    if len(matches) != 1:
+        return None
+    return matches[0]
+
+
 def list_local_keys(collect_errors=False):
     """get list of local keys that appear valid
 
@@ -48,7 +62,7 @@ def list_local_keys(collect_errors=False):
     base_path = os.path.join(os.path.expanduser('~'), '.rerobots')
     likely_keys = []
     if collect_errors:
-        errored_keys = []
+        errored_keys = dict()
     if not os.path.exists(base_path):
         if collect_errors:
             return likely_keys, errored_keys
@@ -73,25 +87,25 @@ def list_local_keys(collect_errors=False):
             if x_perm == 0o600 or x_perm == 0o400:
                 likely_keys.append(x)
             elif collect_errors:
-                errored_keys.append((x, 'unsafe file permissions'))
+                errored_keys[x] = 'unsafe file permissions'
         except jwt.exceptions.DecodeError:
             if collect_errors:
-                errored_keys.append((x, 'coding error'))
+                errored_keys[x] = 'coding error'
         except jwt.exceptions.ImmatureSignatureError:
             if collect_errors:
-                errored_keys.append((x, 'immature signature'))
+                errored_keys[x] = 'immature signature'
         except jwt.exceptions.InvalidIssuerError:
             if collect_errors:
-                errored_keys.append((x, 'invalid issuer'))
+                errored_keys[x] = 'invalid issuer'
         except jwt.exceptions.InvalidAudienceError:
             if collect_errors:
-                errored_keys.append((x, 'invalid audience'))
+                errored_keys[x] = 'invalid audience'
         except jwt.exceptions.ExpiredSignatureError:
             if collect_errors:
-                errored_keys.append((x, 'expired signature'))
+                errored_keys[x] = 'expired signature'
         except:
             if collect_errors:
-                errored_keys.append((x, 'unknown error'))
+                errored_keys[x] = 'unknown error'
     if collect_errors:
         return likely_keys, errored_keys
     else:
